
/*
 * Created on 21-5-2004
 *
 * 
 * 
 */
package dk.kb;

//import java.io.IOException; 
import java.io.File; 
import java.io.BufferedReader;
import java.io.FileReader;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.logging.Logger;   // Enable standard java logging

import java.util.Vector;
import java.util.HashMap;
import java.util.ArrayList;

import java.io.IOException;

import java.io.FileInputStream;
import java.io.ByteArrayOutputStream;

//import java.io.PrintWriter;


/**
 * @author ssc,svc
 *
 *
 */


//////////////////////////////////////////////////
// sample NEDLIB-meta entry 
//////////////////////////////////////////////////

/*
HTTP/1.1 200 OK
Date: Tue, 12 Nov 2002 15:37:33 GMT
Server: Apache/1.3.26 (Unix) PHP/3.0.18 PHP/4.2.3
Last-Modified: Mon, 03 Jun 2002 09:27:45 GMT
ETag: "1b8c7-53e-3cfb3691"
Accept-Ranges: bytes
Content-Length: 1342
Connection: close
Content-Type: text/html
Arc-Length: 1342
Arc-Md5: 26a1b73b624906a681c3f1cfa0179d09
Arc-Depth: 1
Arc-Elapsed-Time: 0.780000
Arc-Url: http://www.voice.dk:80/
Arc-Status: HTTP/1.1 200 OK
Meta-Data: html-tag='title' content='Untitled Document'
Meta-Data: http-equiv='content-type' content='text/html; charset=iso-8859-1'

*/

public class Nedlib {
 
    private HashMap hashMeta = null; // contains all NEDLIB Crawler meta entries
    private HashMap hashHTTP = null; // contains all HTTP header entries
    private StringBuffer data = null; // contains the metafile contents itself
    private File nedlibObject = null;
    private File nedlibMetaObject = null;
    
    public Nedlib(File file){
	hashMeta = new HashMap();
	hashHTTP = new HashMap();
	data = new StringBuffer();
	// datastructure to contain NEDLIB data object

	nedlibObject = new File (file.getAbsolutePath());
	if (!isNedlibFilename(nedlibObject.getName())){
	    System.out.println("WARNING. File " +  file.getAbsolutePath() + " is not a NEDLIB file");
	}
	nedlibMetaObject = new File(nedlibObject.getAbsolutePath() + ".meta");	
	    
    }
	    
    public String getContentFilename(String metaFileName){
    String suffix = ".meta";
    return metaFileName.substring(0, metaFileName.length() - suffix.length());
    }

    public String getHttpHeader(){
	return 	(String) hashMeta.get("HTTP");	
    }

    public StringBuffer getMetaFileContents(){
	return data;
    }

    public ByteArrayOutputStream getArcContents(){

	ByteArrayOutputStream baos = new ByteArrayOutputStream();

      try {	
	String httpHeader = this.getHttpHeader();
	// Add empty line between HTTP header and the object itself
	httpHeader += "\n"; 
	
	FileInputStream contentStream = new FileInputStream(nedlibObject);

	// The next statement encodes the String 'httpheader' into a sequence of bytes using
        // the platform's default charset, storing the result into a new byte array,
        // and finally writes it to a ByteArrayOutputStream

        baos.write(httpHeader.getBytes());

        // read all data from the FileInputStream 'contentStream' and send it 
	// to the ByteArrayOutputStream
      
	int b = contentStream.read();
        while (b!=-1){   // While b != EOF (or maybe rather EOS: End of Stream)
	    baos.write(b);
            b=contentStream.read();
        }	
	// Release ressources to avoid "too many open files" Exception
	contentStream.close(); 

      } catch (IOException e){
	  //TODO
      }	
      return baos;
    } // public ByteArrayOutputStream getArcContents(){

    
    public String getIP(){
        return "0.0.0.0"; // Return Fake IP information
    }

    public String getDate(){
	return (String)hashHTTP.get("Date");
    }

    public long getDateStamp(){
	// TODO: Is this the right thing to do here: Maybe use the Date information from HTTP Header instead	 
         return (nedlibMetaObject.lastModified());
    }

    public String getMimetype(){
	String mimetype = "no-type";
	if (hashHTTP.containsKey("Content-Type")){
	    return (String)hashHTTP.get("Content-Type");
	} else {
	    return new String(mimetype);
	}
    }
    
    // e.g. Arc-Status: HTTP/1.1 200 OK
    // TODO: maybe to be split up into 3 entities protocol, return code, return message
    public String getArcStatus(){
	return (String)hashMeta.get("Arc-Status");
    }


    // e.g. "Arc-Length: 1342"
    public String getArcLength(){
	return (String)hashMeta.get("Arc-Length");
    }

    // e.g. Arc-Url: http://www.voice.dk:80/
    // now with the port removed, if port=80 (as in the above example)

    public String getArcUrl(){
	String arcUrl = (String)hashMeta.get("Arc-Url");
	// if String matches ":80/"
	// replace this substring with just "/"
	String port80 = ":80/";
	int index = arcUrl.indexOf(port80);
	if (index > -1) {	    
	    String n = new String (arcUrl.substring(0,index) +  
				   arcUrl.substring(index+3,arcUrl.length()
						    )
				   );
	    return n;
	} else {
	    return arcUrl;
	}
    }

    // e.g.  "Arc-Md5: 26a1b73b624906a681c3f1cfa0179d09"
    public String getArcMd5(){
   	return (String)hashMeta.get("Arc-Md5");
    }
    // e.g. "Arc-Depth: 1"
    public String getArcDepth(){
	return (String)hashMeta.get("Arc-Depth");
    }
    // e.g. Arc-Elapsed-Time: 0.780000
    public String getArcElapsedTime(){
	return (String)hashMeta.get("Arc-Elapsed-time");
    }
    
    public Vector getMetaData(){ // TBD: Array of strings instead 
	Vector v = new Vector (10,10);
	return v;
    }

    public boolean isNedlibFilename(String filename){
	// check, if filename is 32-character hex-number
	//System.out.println("Checking filename: " + filename);
	Pattern p = Pattern.compile("[a-f0-9]{32}");
	return (p.matcher(filename).find());
    }

    // hashMeta: contains Metadata contained in the NEDLIB .meta file
    //   (after the processing is finished)
    // hashHTTP:  contains HTTP-header contained in the NEDLIB .meta file
    //   (after the processing is finished)
    // data contains the contents of the metafile itself
    //   (after the processing is finished)
    
    public void processMetaFile()
	throws Exception
    {
	// process meta file
        // <Meta-File>   :: <HTTP-Header> <Crawler-info>
        // <Crawler-info>:: <arc-length> <arc-md5> <arc-depth> <arc-elapsedtime> <arc-url> <arc-status>

        BufferedReader metaIn = new BufferedReader( new FileReader(nedlibMetaObject.getAbsolutePath()));

	// Parse HTTP header - skip to first blank line
        String HttpHeader = new String();
	String line = new String();
        String demarcation = "Arc-";

	while ((line=metaIn.readLine()) != null) {
	    //if (line.length()==0) break;     // (I believe, this data is currently never being executed)
	    data.append(line + "\n");
	    if (line.startsWith(demarcation)){ // (This should work, however
		parseLine(line);
		break;
	    }
	    HttpHeader += line + "\n";
            parseLine(line);
	}
	hashMeta.put("HTTP", HttpHeader);
	
	// Parse <Crawler-info> 
	while ((line=metaIn.readLine()) != null) {
	    data.append(line + "\n"); 
	    parseLine(line); // detect  Meta-Data
	    
	}

    }

    /** Parse <id>:<data> and add entry hashMeta[id]=data to hashmap
     *  If <id>="Meta-Data" an arraylist entry is added to hashMeta
     *  as this entry may occur more than once
     *  UPDATES hashMeta
     * @param line - line to parse
     * 
     */

    private void parseLine( String line)
    {
	Pattern p = Pattern.compile(":");
	Matcher m = p.matcher(line);
	if (m.find()){
	    String tagName = line.substring(0,m.start());
	    String tagValue = line.substring(m.end());
	    tagValue= tagValue.trim();
	    if ( (tagName.startsWith("Arc-")) || 
		(tagName.startsWith("Meta-Data"))){
		// insert data into HashMap hashMeta
		if (tagName.equals("Meta-Data")) {
		    if (!hashMeta.containsKey("Meta-Data")) 
			hashMeta.put("Meta-Data",new ArrayList());
		    ArrayList metadataArr = (ArrayList)hashMeta.get("Meta-Data");
		    metadataArr.add(tagValue);
		} 
		else {  // INV: !(tagName.equals("Meta-Data"))
		//assert !hashMeta.containsKey(tagName);
		    hashMeta.put(tagName, tagValue);
		}
	    } else {
		// insert data into HashMap hashHTTP
		hashHTTP.put(tagName, tagValue);
	    }
	}
    }
		
		
		
}
