
/*
 * Created on 6-5-2004
 *
 * 
 * 
 */
package dk.kb;

import java.io.IOException; 
import java.io.File; 
import java.io.FileInputStream;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.ByteArrayOutputStream;
import java.io.PrintWriter;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.logging.Logger;   // Enable standard java logging

// Java classes used for commandline parsing 
import org.apache.commons.cli.Options;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.cli.UnrecognizedOptionException;

import java.util.Vector;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.List;
import org.archive.io.arc.ARCWriter;

// Maybe Use in a later iteration
//import org.archive.io.arc.ARCRecordMetaData;// Store ARC-record metadata
//import org.archive.io.arc.ARCRecord;
// ARC Reference: http://www.archive.org/web/researcher/ArcFileFormat.php


/**
 * @author ssc,svc
 * 
 *
 */

public class NedlibToArc {

    private HashMap settings = null;
    private Vector vectorOfFiles = null;

    // Define String constants
    static String OUTPUT_DIR = "outputDir";
    static String INPUT_DIR = "inputDir";
    static String ARC_PREFIX = "arcPrefix";
    static String COMPRESSED_ARCS = "compressed_arcs";
    static String CONTENT_TYPE = "Content-Type";
    static String HARVEST_INFO_URL = "harvestInfoUrl";

    
    public static void main(String[] args) 
	throws IOException // needed because ARCWriter throws this type of Exception
    {	
	NedlibToArc o = new NedlibToArc();
	o.myMain(args);
    }
        
    public void copyToARC(Vector v, HashMap settings)
	throws IOException // needed because ARCWriter throws this type of Exception
{
    System.out.println("\nEntering copyToARC function");
    
    String prefix = (String) settings.get(ARC_PREFIX);
    String suffix = "-";
    boolean compress = ((Boolean)settings.get(COMPRESSED_ARCS)).booleanValue();
    int maxSize = 100000000; // 100 MbBytes
    
    File destDir = new File ( (String)settings.get(OUTPUT_DIR) );
    List metadata = null; // TODO: decide whether to insert something else than null here
    
    //ARCWriter arcwriter = new ARCWriter(destDir, prefix, suffix, compress, maxSize, metadata);
    // The former constructor is not supported in heritrix-0.6.0
    ARCWriter arcwriter = new ARCWriter(destDir, prefix + suffix, compress, maxSize);

    
   for(int i=0; i < v.size(); i++){
       File contentsFile = (File) v.elementAt(i);
       File metaFile = new File (contentsFile.getAbsolutePath() + ".meta");
	if (!contentsFile.canRead()){
	    System.out.println("WARNING: Contentsfile " + contentsFile.getName() + " is not readable!");
	    if (!contentsFile.exists()) {
		System.out.println("WARNING: contensfile " + contentsFile.getName() + " does not exist!");
	    }
	} else if (!metaFile.canRead()){
	    System.out.println("WARNING: metafile " + metaFile.getName() + " is not readable!");
	    if (!metaFile.exists()) {
		System.out.println("WARNING: metafile " + metaFile.getName() + " does not exist!");
	    }
	} else {
	    try {	    
		//System.out.println("INSERTING INTO ARC");
		System.out.print(".");
	    // load and process meta data    
	    Nedlib nedlibObject = new Nedlib(contentsFile);
	    nedlibObject.processMetaFile();

	    // Load corresponding non meta file into ByteArrayOutputStream
	    
	    ByteArrayOutputStream baos = nedlibObject.getArcContents();
	    
	    // INV: baos contains HTTPheader + the contents itself as a Array of Bytes.
	    
	    String url = nedlibObject.getArcUrl();
	    String ip  = nedlibObject.getIP();
	    long datestamp  = nedlibObject.getDateStamp();
 
	    //long recordLength = (long)hashMeta.get("Arc-Length"); OR baos.size();
	    String mimetype = nedlibObject.getMimetype();

	    // Prototype for writing to the ARCWriter:

	    // arcwriter.write(String uri, == Arc-Url
	    //   String contentType, [Taken from Content-Type in HTTP-header block]
	    //   String HostIP, == "0.0.0.0" (not recorded in the NEDLIB archive)
	    //   long fetchBeginTimestamp  == archival time (not Recorded in NEDLIB archive)
	    //            " == "Date: Tue, 01 Oct 2002 00:02:46 GMT" convert properly
	    //
	    //   long recordLength, [== Arc-Length (?)]
	    //   ByteArrayOutputStream baos
	    // )

	    arcwriter.write(url,mimetype, ip,datestamp, baos.size(), baos);

      	    int urlsize = baos.size();

	    // create Metadata section
	  
	    url += "/NEDLIB-meta.txt";
	
	    mimetype = "text/plain";
	    
	   // Now save the nedlib metadata file
	    // create dummy HTTP response header to ensure correct display of actual data
	    String nedlibMetaFile = "HTTP/1.1 200 OK\nContent-Type: text/plain\n\n";
	    nedlibMetaFile += nedlibObject.getMetaFileContents().toString();
	    ByteArrayOutputStream baos2 = new ByteArrayOutputStream();
	    baos2.write(nedlibMetaFile.getBytes());
	    
	    arcwriter.write(url,mimetype, ip,datestamp, baos2.size(), baos2);

	    } catch (Exception ee) {
		ee.printStackTrace(System.out);
	    }
	} // end else
    } // end for

   arcwriter.close(); // close all open ARC-files
}

    private Options makeOptionList(){
	Options options = new Options();
	options.addOption(new Option("h","help", false,
            "Prints this message and exits."));
        options.addOption(new Option("u","info Url", true,
            "Url pointing to information about this harvest.  Default: \"\"."));
        options.addOption(new Option("o", "output dir", true,
            "Directory where the output ARC files are stored." +
            " Default: Current working directory (.)."));
        options.addOption(new Option("c", "compress mode", false,
            "Generate gzip compressed ARC files. Default is not-compressed"));
	options.addOption(new Option("i", "input dir", true,
            "Directory where we search for NEDLIB archive files." +
            " Default: Current working directory (.)."));
	options.addOption(new Option("p", "prefix name", true,
            "Choice of prefix on the output ARC files." +
            " Default: NEDLIB"));
	return options;
    }


public void printUsage(String err, Options os) {
    String help = new String();
    Option o;
    if (err.length() > 0)
	System.out.println("Error: " + err);     
    if (os.hasOption("h")){
	o = os.getOption("h");
	help += "-h  (" + o.getDescription() + ")";
    }	
    if (os.hasOption("c")){
	o = os.getOption("c");
	help += "-c  (" + o.getDescription() + ")";
    }	

    //System.out.println("HELP: " + help);
    help = new String("Usage: java dk.kb.NedlibToArc");
    help += "\n\t-h\t\tPrint out this information";
    help += "\n\t-i <inputdir>\tDirectory where we look for nedlib documents. Default is Current Working Directory";
    help += "\n\t-o <outputdir>\tDirectory where we put the output ARC files. Default is Current Working Directory";;
    help += "\n\t-p <prefix>\tSelect a prefix for the output ARC files. Default is NEDLIB";
    help += "\n\t-c\t\tChoose compressed ARC files. Default is uncompressed";
    System.out.println(help);
    
    }

public Vector collectFiles(File dir, Vector v){

    // Check that filepath dir exists
    // It is readable;

    if (dir.exists() == false){
        System.out.println("WARNING: File/Dir " + dir.getName() + " does not exist!");
        return v;
    }

    // Post-condition: File/Dir dir exists
    if (dir.canRead() == false){
        System.out.println("WARNING: File/Dir " + dir.getName() + " is not readable!");
        return v;
    }

    // Post-condition: File/Dir dir is readable
    if (dir.isDirectory()){
        File[] list = dir.listFiles();
        // System.out.println("\nDirectory '" + dir.getName() + "' contains " + list.length + " files");
        for(int i=0; i < list.length; i++){
            // make a recursive call here to treat each of the directories separately
            // System.out.println("\nRecurse into sub-Directory '" + list[i].getName());
            v = collectFiles(list[i], v);
        }
    } else { // dir is a normal file
        if (!dir.getName().endsWith(".meta")){
	    // check also, if filename is a 32-character hex-number
	    if (isNedlibFilename(dir.getName())){
		System.out.println("Adding " + dir.getAbsolutePath() + " to vector");
		v.add(new File(dir.getAbsolutePath()));
	    }
	    
	}
    }
    return v;
} // END OF collectFiles()

    // Copy of function in Nedlib.java
    // could not access the isNedlibFilename in class Nedlib

    public boolean isNedlibFilename(String filename){
        // check, if filename is 32-character hex-number
	Pattern p = Pattern.compile("[a-f0-9]{32}");
        return (p.matcher(filename).find());
    }




    public void myMain(String[] args)
	throws IOException // needed because ARCWriter throws this type of Exception
    { 
	// The commandline parsing is based on code from the Heritrix project 
	
        Options options = makeOptionList();
	CommandLine commandLine = null;
	File outputDir;
	File inputDir;
        HashMap settings = new HashMap();
	boolean compressed_arcs = false;
	String prefix;
	String harvestInfoUrl = null; // "http://www.netarkivet.dk/kb/harvests/2002/uge46/harvest.html"; 
	PosixParser parser = new PosixParser();
        try
        {
            commandLine = parser.parse(options, args, false);
        }

        catch (UnrecognizedOptionException e)
        {
            printUsage(e.getMessage(),options);
        }
	catch (ParseException e)
	{
	    System.out.println(e.getMessage());
	}
	catch (NullPointerException e)
	{
	    System.out.println(e.getMessage());
	}

	// INV: All options recognized correctly

	// Now set system variables according to information parsed above
	// and 

	if (commandLine.hasOption('h')){ // If -h is used, print Usage and exit program
	    printUsage("",options);
	    return;
	}
   
        /*   	

	if (commandLine.hasOption('u')) { // set harvestInfoUrl
	    harvestInfoUrl = commandLine.getOptionValue('u');
	} else {
	    harvestInfoUrl = ""; // Default harvestInfoUrl
	}
	settings.put(HARVEST_INFO_URL,new String(harvestInfoUrl));

	*/

	if (commandLine.hasOption('o')) { // set Output directory
	    outputDir = new File (commandLine.getOptionValue('d'));
	} else {
	    outputDir = new File ("."); // Default outputdir = CWD
	}
	
	settings.put(OUTPUT_DIR, new String (outputDir.getAbsolutePath()) );
	

	if (commandLine.hasOption('i')) { // set Input directory
	    inputDir = new File (commandLine.getOptionValue('i'));
	} else {
	    inputDir = new File ("."); // Default inputdir = CWD
	}
   
        settings.put(INPUT_DIR, new String(inputDir.getAbsolutePath()));
		     
	if (commandLine.hasOption('c')) { // The User wants to have compressed ARC files
	     compressed_arcs = true;
	} else {
	     compressed_arcs = false; // Default behaviour: No compression
	}

	if (commandLine.hasOption('p')) { // Which prefix to put on the ARC files
	    prefix = new String (commandLine.getOptionValue('p'));
	} else {
	    prefix = new String ("NEDLIB"); // Default prefix is NEDLIB  
	}

	settings.put(ARC_PREFIX,new String(prefix));
	settings.put(COMPRESSED_ARCS,new Boolean(compressed_arcs));
      
	// INV: All arguments have now been processed

	System.out.println("Traversing filepath(" + inputDir.getName() + 
			   ") to find nedlib data");
	    
	Vector v= collectFiles(inputDir, new Vector(10,10));
	if (v != null) {
	    // Print number of Nedlib objects to export
	    System.out.println("Nedlib objects found in " + inputDir.getAbsolutePath() + ": " + v.size());
	  
	    // Export Nedlib data to ARC
	    if (!v.isEmpty()){
		copyToARC(v,settings);
		System.out.println("Finished exporting to ARC");
	    } else {
		System.out.println("No files was exported to ARC!");
	    }
	}  else {
	    System.out.println("Unable to ascertain number of files to be exported");
            System.out.println("Aborting program");
	}
        return;
    }

}

